<?php
/*
Plugin Name: PostMaster
Plugin URI: http://blog.xforward.com/?page_id=65
Description: The PostMaster plugin allows you to make posts-by-email with multimedia attachments. It processes all incomming email-posts,extracts the attachments, and embeds them into the post body. It currently supports image and mobile video (3g2) MIME types,with more to come in the future (hopefully). Please note that version 2.0 only supports a single attachment.
Version: 2.0
Author: Brett Duncavage
Author URI: http://xforward.com
*/

/*  
Copyright 2007  Brett Duncavage (email : brett@xforward.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// important that this plugin is in standard install directory, i.e. wp-content/plugins/pluginname
require(dirname(__FILE__).'/../../../wp-config.php');
// add the admin options page stuff
require_once('pm_admin_menu.php');
// only import the graphics functions if needed

if(get_option('pm_resize_enabled') == 'on') {
	require_once('pm_graphics.php');
}

add_filter('phone_content', 'pm_process_post');
add_filter('category_save_pre', 'pm_process_category');
add_filter('title_save_pre', 'pm_clean_post_title');
add_action('publish_phone', 'pm_auto_publish');

function pm_auto_publish($postID) {
	// for WordPress 2.5
	// email posts not automatically published
	if(get_option('pm_auto_publish_enabled') == 'on') {
		wp_publish_post($postID);
	}
}

function pm_process_category($cats) {
	if(get_option('pm_assign_cats_enabled') == 'on') {
		$catsList = get_option('pm_temp_cats_list');
		if(isset($catsList) && $catsList != '') {
			$catsArray = explode(',', $catsList);
			$idx = 0;
			foreach($catsArray as $catName) {
				$catId = get_cat_ID($catName);
				if($catId != FALSE) {
					$catIdsToAdd[$idx] = $catId;
					$idx++;
				}
			}
		} else {
			$catIdsToAdd = $cats;
		}
		update_option('pm_temp_cats_list', '');
	} else {
		$catIdsToAdd = $cats;
	}
	return $catIdsToAdd;
}

function pm_clean_post_title($title) {
	$title = trim($title);
	if(get_option('pm_assign_cats_enabled') == 'on') {
		$title_words = explode(' ', $title);
		foreach($title_words as $word) {
			if(strstr($word, '%cats=')) {
				$cats = $word;
			} else {
				$new_title = $new_title.' '.$word;
			}
		}
		if(isset($cats)) {
			$first_split = explode('=', $cats);
			update_option('pm_temp_cats_list', $first_split[1]);
		}
	} else {
		$new_title = $title;
	}

	return $new_title;
}

function pm_doResolveImageFormat($buffer) {
	$result = "jpg"; // the default will be jpg because that's pretty common
	// just try to detect the mime type, crude? yes
	if(strstr($buffer, "image/jpeg") || strstr($buffer, "image/jpg")) {
		$result = "jpg";
	} else if(strstr($buffer, "image/png")) {
		$result = "png";
	} else if(strstr($buffer, "image/gif")) {
		$result = "gif";
	} else if(strstr($buffer, "image/bmp")) {
		$result = "bmp";
	} else if(strstr($buffer, "image/tiff")) {
		$result = "tiff";
	}
	return $result;
}

function pm_process_post($raw_content) {
	$exploded_content = explode("\r\n", $raw_content);
	$line_counter = 0;


	if(get_option('pm_debug_enabled') == 'on') {
		echo $raw_content;
	}

	// PostMaster credit sig
	$pm_credit_sig = '<p>&raquo;This post was processed by <a href="http://xforward.com">PostMaster</a></p>';

	// test for multi-part, basically if Content-Type is missing from content, then we've
	// got a vanilla post
	if(stristr($raw_content, 'Content-Type:') == FALSE) {
		// need to clean the content of tags
		return strip_tags($raw_content);
	}

	foreach($exploded_content as $line) {
		// find the different content parts
		if (stristr($line, "Content-Type: text/plain") != FALSE or stristr($line, "Content-Type: text/html") != FALSE) {
			// 2.0 rewrite
			// Ahhhh, that's better! 
			// Cleaner, more reliable, and should work with 
			// almost all mail clients and services.
			//

			// find first and second lines with 0 length
			$vzw_counter = 0;
			$first_line = -1;
			$second_line = -1;
			foreach($exploded_content as $vzw_line) {
				if(strlen($vzw_line) == 0 && $first_line == -1) {
					$first_line = $vzw_counter;
				} else if(strlen($vzw_line) == 0 && ($second_line == -1 && $first_line > -1)) {
					$second_line = $vzw_counter;
				}
				$vzw_counter++;
			}
			// now grab all the lines in between
			for($i = $first_line; $i < $second_line; $i++) {
				if($i + 1 == $second_line) {
					$text_content = $text_content.$exploded_content[$i].'<br/>';
				} else {
					$text_content = $text_content.$exploded_content[$i];
				}
			}

			$temp_text_content = pm_clean_text_content($text_content, 'helio', $exploded_content);
			
			//echo "<hr/>TEMP_TEXT_CONTEXT<br/>".$temp_text_content.'<hr/>';

			if(strlen($temp_text_content) == 0 || strstr($temp_text_content, 'Content-Type:')) {
				// try the next fix
				$temp_text_content = pm_clean_text_content($text_content, 'att_centro', $exploded_content);
			}
			if(strlen($temp_text_content) > 0) {
				$text_content = $temp_text_content;
			}
			// clean the text content of tags
			$text_content = strip_tags($text_content, '<em><a><br><b>');

		} else if(stristr($line, "Content-Type: image/") != FALSE || stristr($line, "Content-Type: application/octet-stream") != FALSE) {
			// grab the binary and save it.
			$bin_content_begin = stristr($raw_content, "Content-Type: image/");

			$media_type = 'image';
			if($bin_content_begin == FALSE) {
				$bin_content_begin = stristr($raw_content, 'Content-Type: application/octet-stream');
				$media_type = 'video';
			}
			$bin_content_begin = strstr($bin_content_begin, "\r\n\r\n");
			$bin_content_begin = substr($bin_content_begin, 4);
			$bin_content_end_pos = strpos($bin_content_begin, "\r\n\r\n");

			if($bin_content_end_pos == FALSE) {
				// let's try a different delimiter
				$bin_content_end_pos = strpos($bin_content_begin, "--"); // not the best thing to use :D
			}
			$bin_content = substr($bin_content_begin, 0, $bin_content_end_pos);

			// now i *assume* this is base64 encoded, this might not always work
			// someday i'll actually detect the encoding used
			$raw_bin = base64_decode($bin_content);

			// get the name of the image
			$file_name_offset = 0;
			$file_name_begin = stristr($raw_content, "Content-Type: image/");
			if($file_name_begin == FALSE) {
				$file_name_begin = stristr($raw_content, 'Content-Type: application/octet-stream');
			}
			$file_name_begin = stristr($file_name_begin, "filename=\"");
			// not all systems use '"' in filename
			if($file_name_begin == FALSE) {
				//$file_name_begin = stristr($raw_content, "Content-Type: image/");
				$file_name_begin = stristr($raw_content, "filename=");
				$file_name_end_pos = strpos($file_name_begin, "\r\n");
				$file_name_offset = 9;
			} else {
				$file_name_end_pos = strpos($file_name_begin, "\"\r\n");
				$file_name_offset = 10;
			}
			$file_name = strtolower(substr($file_name_begin, $file_name_offset, $file_name_end_pos - $file_name_offset));
			
			if(get_option('pm_rename_attachment') == 'on') {
				// genterate a new filename with timestamp
				// not using milliseconds in order to keep compatibility with PHP4
				$date_time_str = date('m-d-Y-his');
				// get the extension from the filename
				// 1.0.4 doesn't work with blackberry
				if(count($exts) > 1) {
					$exts = explode('.', $file_name);
					$exts = $exts[count($exts) - 1];
					$file_name = $date_time_str.'.'.$exts;
				} else {
					// do the blackberry fix
					$format_ext = pm_doResolveImageFormat($line);
					$file_name = $date_time_str.'.'.$format_ext;
				}
				$save_name = get_option('pm_attachment_dir').'/'.$file_name;
				// 1.0.3 make sure file doesn't already exist!
				for($fname_idx = 0; file_exists($save_name); $fname_idx++){
					$file_name = sprintf('%s_%d.%s', $date_time_str, $fname_idx, $format_ext); // returns a string?!
					$save_name = get_option('pm_attachment_dir').'/'.$file_name;
				}
			} else {
				// tack on the user-defined path
				$save_name = get_option('pm_attachment_dir').'/'.$file_name;
			}

			// replacing file_put_contents with fopen and fwrite, backwards compatible with PHP4
			//$result = file_put_contents($save_name, $raw_bin);
			$result = TRUE;
			if($save_name[0] != '/') {
				// relative paths not working??
				$save_name = dirname(__FILE__).'/'.$save_name;
			}
			
			$fp = fopen($save_name, 'x');
			if($fp != FALSE) {
				$result = fwrite($fp, $raw_bin);
				fclose($fp);

				// do the file resizing shite
				if(get_option('pm_resize_enabled') == 'on') {
					$new_width = get_option('pm_resize_width');
					$new_height = get_option('pm_resize_height');
					$keep_orig_image = get_option('pm_keep_orig_image');
					// pulling this out, not really worth the headache.
					//$thumb_dir = get_option('pm_thumb_dir');
					
					if($keep_orig_image == 'off') {
						$resize_name = $save_name;
					} else {
						$save_name_parts = explode('.', $save_name);
						$the_save_path = "";
						// FIX for blog.wetty.de
						// there could've been dots in some of the directory names so let's make sure that
						// we only grab the extension then smash everything else back together
						if(count($save_name_parts) > 2) {
							$nParts = count($save_name_parts);
							$the_extension = $save_name_parts[$nParts - 1];
							// now put everything before the extension back together
							for($i = 0; $i < $nParts - 1; $i++) {
								if(strlen($the_save_path) > 0) {
									$the_save_path = $the_save_path.'.'.$save_name_parts[$i];
								} else {
									$the_save_path = $save_name_parts[$i];
								}
							}
						} else {
							$the_extension = $save_name_parts[1];
							$the_save_path = $save_name_parts[0];
						}

						$file_name_parts = explode('.', $file_name);
						$resize_name = $the_save_path.'_resize.'.$the_extension;
						// reset file_name so the correct image is referenced in the post
						$file_name_resize = $file_name_parts[0].'_resize.'.$file_name_parts[1];
					}
					if($new_width > 0 && $new_height > 0) {
						echo 'Resizing image to '.$new_width.'x'.$new_height.'<br/>';
						echo 'Saving resized image as: '.$resize_name.'<br/>';
						pm_resizeimage($format_ext, $new_width, $new_height, $save_name, $resize_name);
					}
				}
			}

			// cleanup the path
			$pm_attachment_dir = get_option('pm_attachment_dir');
			$base_path = ABSPATH;
			$base_path_pos = strstr($pm_attachment_dir, $base_path);
			if($base_path_pos != FALSE) {
				$attachment_dir = substr(get_option('pm_attachment_dir'), strlen($base_path));
			} else {
				// relative path stick plugin dir before it
				$plugin_dir = substr(dirname(__FILE__), strlen($base_path));
				$attachment_dir = $plugin_dir.'/'.get_option('pm_attachment_dir');
				//$thumb_dir = $plugin_dir.'/'.get_option('pm_thumb_dir');
			}
			if($file_name_resize == FALSE) {
				$media_url = get_option('siteurl').'/'.$attachment_dir.'/'.$file_name;
				$media_url_resize = $media_url;
			} else {
				$media_url_resize = get_option('siteurl').'/'.$attachment_dir.'/'.$file_name_resize;
				$media_url = get_option('siteurl').'/'.$attachment_dir.'/'.$file_name;
			}
			echo 'Save name: '.$save_name.'<br/>';
			if($resize_name) {
				echo 'Resized name: '.$resize_name.'<br/>';
			}
			echo 'Attachment url: '.$media_url.'<br/>';
			if($media_type == 'image') {
				$post_template = get_option('pm_image_post_template');
				$img_w = 0;
				$img_h = 0;

				if($resize_name == FALSE) {
					$img_props = getimagesize($save_name);
				} else {
					$img_props = getimagesize($resize_name);
				}

				if($img_w == 0) {
					$img_w = $img_props[0];
				}
				
				if($img_h == 0) {
					$img_h = $img_props[1];
				}
				
				// put the values in the post
				$final_post = str_replace('%img_url%', $media_url, $post_template);
				$final_post = str_replace('%img_url_resize%', $media_url_resize, $final_post);
				$final_post = str_replace('%img_w%', $img_w, $final_post);
				$final_post = str_replace('%img_h%', $img_h, $final_post);
				$final_post = str_replace('%post_text%', $text_content, $final_post);

			} else if($media_type == 'video') {
				$post_template = get_option('pm_video_post_template');
				$final_post = str_replace('%video_url%', $media_url, $post_template);
				$final_post = str_replace('%post_text%', $text_content, $final_post);
			}
			
			if(get_option('pm_credit_flag') == 'on') {
				$final_post = $final_post.$pm_credit_sig;
			}
		}
	}
	if(!isset($final_post)) {
		// no image to post
		$final_post = $text_content;
	}

	return $final_post;
}

function pm_clean_text_content($text_content, $fix_to_try, $exploded_content) {
	$new_text_content = $text_content;
	// try helio-gmail fix
	if($fix_to_try == 'helio' && strstr($text_content, 'Content-Type:')) {
		$strip_from = strpos($text_content, '------=');
		$new_text_content = substr($text_content, 0, $strip_from);
	} else if($fix_to_try == 'att_centro' && strstr($text_content, 'Content-Type:')) {
		// need to regrab the blank lines, ingoring the first set
		$text_content = "";

		$vzw_counter = 0;
		$first_line = -2;
		$second_line = -1;
		foreach($exploded_content as $vzw_line) {
			if(strlen($vzw_line) == 0 && $first_line == -2) {
				$first_line = -1;
			} 
			else if(strlen($vzw_line) == 0 && $first_line == -1) {
				$first_line = $vzw_counter;
			}
			else if(strlen($vzw_line) == 0 && ($first_line > -1 && $second_line == -1)) {
				$second_line = $vzw_counter;
			}
			$vzw_counter++;
		}

		// now grab all the lines in between
		for($i = $first_line; $i < $second_line; $i++) {
			
			if($i + 1 == $second_line) {
				$text_content = $text_content.$exploded_content[$i].'<br/>';
			} else {
				$text_content = $text_content.$exploded_content[$i];
			}
		}
		$new_text_content = $text_content;
	}

	return $new_text_content;
}

?>
